######################################################################
## First part of Example 2 in Zeng, He, and Zhu (2010)
######################################################################

rm(list = ls())
source("sim-lasso.R")
dyn.load("sim-lasso-clib.dll")
## dyn.load("sim-lasso-clib.so")   ### if Unix/Linux or MacOS

h0 = 0.49
lda.seq = seq(0, 1.0, by = 0.01)
KK = length(lda.seq)

######################################################################

n = 100; p = 20; iter = 200
b = c(1, -1, 2, -0.5, rep(0, 16))
yfun = expression( x%*%b + 4 * sqrt(abs(x%*%b + 1)) + e )

lambda.mat = matrix(0, nrow = iter, ncol = 2)
colnames(lambda.mat) = c("opt", "upper")
cvmean = array(0, dim = c(iter, KK))
cvsd   = array(0, dim = c(iter, KK))

beta.mat = array(0, dim = c(iter, KK, p))
angle.mat = array(0, dim = c(iter, KK))

time1 = date()
for(i in 1:iter)
{
  cat("|<===> i =", i, "<===>", date(), "\n")

  x = matrix(runif(n * p, -2, 2), ncol = p)
  e = rnorm(n)
  y = eval(yfun)

  for(k in 1:KK)
  {
    ans = sim.lasso(y, x, h0, lda.seq[k])
    beta.mat[i, k, ] = ans$beta 
    angle.mat[i, k] = dist.vec(ans$beta, b) 
  }

  ans = choose.lambda(y, x, h0, lda.seq, m = 10)
  cvmean[i, ] = ans$cvmean
  cvsd[i, ] = ans$cvsd
  lambda.mat[i, 1] = ans$opt
  lambda.mat[i, 2] = ans$upper
}
time2 = date()

dump(c("yfun", "n", "p", "iter", "time1", "time2", "b",
       "h0", "lda.seq", "lambda.mat", "cvmean", "cvsd", "beta.mat", "angle.mat"),
       file = "example-2-a.Rdata")

### figure 2

pos = apply(angle.mat, 1, which.min)
lambda.oracle = lda.seq[pos]

lambda.cv = lambda.mat[, 1]

d1 = density(lambda.oracle)
d2 = density(lambda.cv)

plot(d1, xlab = "", ylab = "", main = "", col = 1, lty = 1,
     xlim = range(c(d1$x, d2$x)), ylim = range(c(d1$y, d2$y)))
mtext("optimal lambda", side = 1, line = 2.5)
mtext("density", side = 2, line = 2.5)
lines(d2, col = 2, lty = 2)
legend(max(d1$x, d2$x), max(d1$y, d2$y),
       c("theta known", "ten-fold cv"), col = c(1, 2), lty = c(1, 2), xjust = 1)

######################################################################
## END OF THE FILE
######################################################################
